@@ -1,4 +1,5 @@
-.onLoad <- function(lib, pkg) { # nocov start
+.onLoad <- function(lib, pkg) {
+  # nocov start
   cred_funs_set_default()
   invisible()
 } # nocov end
