@@ -7,7 +7,7 @@ f <- function(ssid = deaths_id, i = 0) {
     "sheets.spreadsheets.get",
     params = list(spreadsheetId = ssid, fields = "spreadsheetId")
   )
-  raw_resp <- googlesheets4::request_make(req)
+  raw_resp <- gargle::request_make(req)
   code <- httr::status_code(raw_resp)
   cat(i, "code:", code, "\n")
   if (code >= 200 && code < 300) {
@@ -30,7 +30,10 @@ resp
 stopifnot(httr::status_code(resp) == 429)
 saveRDS(
   gargle:::redact_response(resp),
-  testthat::test_path("fixtures", "sheets-spreadsheets-get-quota-exceeded-readgroup_429.rds"),
+  testthat::test_path(
+    "fixtures",
+    "sheets-spreadsheets-get-quota-exceeded-readgroup_429.rds"
+  ),
   version = 2
 )
 
